# Copyright 2015 Johannes Grassler <johannes@btw23.de>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import re

class ExcludeFilter(object):
    """Removes selected mkdown_include include statements (useful for excluding
    a macros include pulled in by every chapter)"""

    def __init__(self, **kwargs):
        self.exclude = kwargs.get('exclude', [])

    def run(self, lines):
        """Filter method"""
        ret = []
        for line in lines:
            for exclude in self.exclude:
                line = re.sub(r'\{!%s!\}' % exclude, '', line)
            ret.append(line)

        return ret



# Copyright 2015 Johannes Grassler <johannes@btw23.de>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import print_function
import os
import re


class ImageFilter(object):
    """Filter for adjusting image targets (absolute file names, optionally
    different extensions"""
    def __init__(self, **kwargs):
        self.filename = kwargs.get('filename', None)
        self.image_path = kwargs.get('image_path', None)
        self.adjust_path = kwargs.get('adjust_path', True)
        self.image_ext = kwargs.get('image_ext', None)

    def run(self, lines):
        """Filter method"""
        # Nothing to do in this case
        if (not self.adjust_path) and (not self.image_ext):
            return lines

        ret = []

        for line in lines:
            processed = {}
            while True:
                alt = ''
                img_name = ''

                match = re.search(r'!\[(.*?)\]\((.*?)\)', line)

                # Make sure there is in fact an image file name
                if match:
                    # Skip images we already processed
                    if processed.has_key(match.group(0)):
                        break
                    alt = match.group(1)
                    img_name = match.group(2)
                else:
                    break

                if self.image_ext:
                    img_name = re.sub(r'\.\w+$', '.' + self.image_ext, img_name)

                if self.adjust_path and (self.image_path or self.filename):
                    # explicitely specified image path takes precedence over
                    # path relative to chapter
                    if self.image_path and self.filename:
                        img_name = os.path.join(
                                os.path.abspath(self.image_path),
                                os.path.dirname(self.filename),
                                img_name)

                    # generate image path relative to file name
                    if self.filename and (not self.image_path):
                        img_name = os.path.join(
                                os.path.abspath(
                                    os.path.dirname(self.filename)),
                                img_name)
                line = re.sub(r'!\[(.*?)\]\((.*?)\)',
                        '![%s](%s)' % (alt, img_name), line)

                # Mark this image as processed
                processed[match.group(0)] = True

            ret.append(line)

        return ret



# Copyright 2015 Johannes Grassler <johannes@btw23.de>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Wrapper for using markdown.markdown_include as simple preprocessor (just
pulls in includes without running the HTML generator)"""

from __future__ import print_function
import markdown_include.include as incl


### This class is merely a wrapper for providing markdown_include.include
class IncludeFilter(incl.IncludePreprocessor):
    def __init__(self, **kwargs):
        self.base_path = kwargs.get('base_path', '.')
        self.encoding = kwargs.get('encoding', 'utf-8')



# Copyright 2015 Johannes Grassler <johannes@btw23.de>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

class ChapterheadFilter(object):
    """Filter for adding chapter titles from mkdocs.yml to chapter files"""

    def __init__(self, **kwargs):
        self.headlevel = kwargs.get('headlevel', 1)
        self.title = kwargs.get('title', None)
        if self.title == None:
            raise ValueError(
                    'Mandatory keyword argument `title` missing.')

    def run(self, lines):
        """Filter method"""

        head = [('#' * self.headlevel) + ' ' + self.title, '']

        head.extend(lines)

        return head



# Copyright 2015 Johannes Grassler <johannes@btw23.de>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import re

# TODO: Implement working cross-references (for now they are simply replaced by
#       their link titles).

class XrefFilter(object):
    """Replaces mkdocs style cross-references by just their title"""

    def run(self, lines):
        """Filter method"""
        ret = []
        for line in lines:
            while True:
                match = re.search(r'\[(.*?)\]\((.*?\.md)\)', line)
                if match != None:
                    title = match.group(1)
                    line = re.sub(r'\[.*?\]\(.*?\.md\)', title, line, count=1)
                else:
                    break
            ret.append(line)

        return ret



#!/usr/bin/env/python

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

long_description = (
    "mkdocs_pandoc is a library of preprocessors that convert mkdocs style markdown "
    "(multiple files, with the document structure defined in the mkdocs "
    "configuration file mkdocs.yml) into a single markdown document digestible by "
    "pandoc. It ships with the command line frontend tool mkdocs2pandoc as its primary " 
    "user interface."
    )

setup(
    name='mkdocs_pandoc',

    version='0.2.0',

    description='A translator from mkdocs style markdown to pandoc style '
                + 'markdown',

    long_description=long_description,

    url='https://github.com/jgrassler/mkdocs-pandoc',
    author='Johannes Grassler',
    author_email='johannes@btw23.de',
    license='Apache',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Topic :: Documentation',
        'Topic :: Text Processing',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
    ],

    keywords='mkdoc markdown pandoc',
    packages=find_packages(),

    install_requires=['mkdocs>=0.14.0',
            'markdown-include>=0.5.1'
            ],

    entry_points={
        'console_scripts': [
            'mkdocs2pandoc=mkdocs_pandoc.cli.mkdocs2pandoc:main',
        ],
    },
)
